export interface GhostErrorOptions {
    message?: string;
    statusCode?: number;
    level?: string;
    id?: string;
    context?: string;
    help?: string;
    errorType?: string;
    errorDetails?: any;
    code?: string;
    property?: string;
    redirect?: string;
    hideStack?: boolean;
    err?: Error | string;
}
export declare class GhostError extends Error {
    statusCode: number;
    errorType: string;
    level: string;
    id: string;
    context?: string;
    help?: string;
    errorDetails: any;
    code: string | null;
    property: string | null;
    redirect: string | null;
    hideStack: boolean;
    constructor(options?: GhostErrorOptions);
}
//# sourceMappingURL=GhostError.d.ts.map