import * as ghostErrors from "./errors";
import { deserialize, isGhostError, prepareStackForUser, serialize } from "./utils";
export * from "./errors";
var src_default = ghostErrors;
const utils = {
  serialize,
  deserialize,
  isGhostError,
  prepareStackForUser
};
export {
  src_default as default,
  utils
};
